use proc_macro::TokenStream;
use quote::quote;
use syn::{parse_macro_input, DeriveInput, FieldsNamed};

#[proc_macro_derive(Reflection)]
pub fn reflection(item: TokenStream) -> TokenStream {
    let  DeriveInput {ident, data, ..}= parse_macro_input!(item);

    if let syn::Data::Struct(s) = data {
        if let syn::Fields::Named(FieldsNamed{ named, .. }) = s.fields {
            let idents = named.iter().map(|f| f.ident.as_ref())
                .filter_map(|f| f);

            let types = named.iter().map(|f| &f.ty);

            let output = quote! {
                impl Reflection for #ident {
                    fn name(&self) -> &str {
                        stringify!(#ident)
                    }

                    fn fields(&self) -> &[(&str, &str)] {
                       &[#((stringify!(#idents), stringify!(#types))),*]
                    }
                }
            };

            output.into()
        } else {
            let output = quote! {
                impl Reflection for #ident {
                    fn name(&self) -> &str {
                        stringify!(#ident)
                    }

                    fn fields(&self) -> &[(&str, &str)] {
                       &[]
                    }
                }
            };

            output.into()
        }
    } else {
        panic!("Cannot apply attribute to non structs.");
    }
}